const OFF   = 0;    // OFF   or 0 - turn the rule off
const WARN  = 1;    // WARN  or 1 - turn the rule on as a warning (does not affect exit code)
const ERROR = 2;    // ERROR or 2 - turn the rule on as an error (exit code will be 1)

const MAX_COMPLEXITY = 20;
const MAX_CLASSES = 1;


module.exports = {
  parserOptions: {
    ecmaVersion: '2022',
    sourceType: 'module'
  },
  rules: {
    // enforce getter/setter pairs in objects
    // https://eslint.org/docs/rules/accessor-pairs
    'accessor-pairs': [OFF, {
      setWithoutGet: false,
      getWithoutSet: false,
      enforceForClassMembers: false
    }],

    // enforce return statements in callbacks of array's methods
    // https://eslint.org/docs/rules/array-callback-return
    'array-callback-return': [ERROR, {
      allowImplicit: true,
      checkForEach: false
    }],

    // treat var statements as if they were block scoped
    // https://eslint.org/docs/rules/block-scoped-var
    'block-scoped-var': ERROR,

    // specify the maximum cyclomatic complexity allowed in a program
    // https://eslint.org/docs/rules/complexity
    complexity: [OFF, {
      max: MAX_COMPLEXITY
    }],

    // enforce that class methods use "this"
    // https://eslint.org/docs/rules/class-methods-use-this
    'class-methods-use-this': [ERROR, {
      exceptMethods: []
    }],

    // require return statements to either always or never specify values
    // https://eslint.org/docs/rules/consistent-return
    'consistent-return': [ERROR, {
      treatUndefinedAsUnspecified: false
    }],

    // specify curly brace conventions for all control statements
    // https://eslint.org/docs/rules/curly
    curly: [ERROR, 'multi-line'],

    // require default case in switch statements
    // https://eslint.org/docs/rules/default-case
    'default-case': [ERROR, {
      commentPattern: '^no default$'
    }],

    // enforce default clauses in switch statements to be last
    // https://eslint.org/docs/rules/default-case-last
    'default-case-last': ERROR,

    // enforce function default parameters to be last
    // https://eslint.org/docs/rules/default-param-last
    'default-param-last': ERROR,

    // encourages use of dot notation whenever possible
    // https://eslint.org/docs/rules/dot-notation
    'dot-notation': [ERROR, {
      allowKeywords: true
    }],

    // enforce consistent newlines before or after dots
    // https://eslint.org/docs/rules/dot-location
    'dot-location': [ERROR, 'property'],

    // require the use of === and !==
    // https://eslint.org/docs/rules/eqeqeq
    eqeqeq: [ERROR, 'always', {
      null: 'ignore'
    }],

    // require grouped accessor pairs in object literals and classes
    // https://eslint.org/docs/rules/grouped-accessor-pairs
    'grouped-accessor-pairs': [ERROR, 'anyOrder'],

    // make sure for-in loops have an if statement
    // https://eslint.org/docs/rules/guard-for-in
    'guard-for-in': ERROR,

    // enforce a maximum number of classes per file
    // https://eslint.org/docs/rules/max-classes-per-file
    'max-classes-per-file': [ERROR, MAX_CLASSES],

    // disallow the use of alert, confirm, and prompt
    // https://eslint.org/docs/rules/no-alert
    'no-alert': WARN,

    // disallow use of arguments.caller or arguments.callee
    // https://eslint.org/docs/rules/no-caller
    'no-caller': ERROR,

    // disallow lexical declarations in case/default clauses
    // https://eslint.org/docs/rules/no-case-declarations
    'no-case-declarations': ERROR,

    // disallow returning value in constructor
    // https://eslint.org/docs/rules/no-constructor-return
    'no-constructor-return': ERROR,

    // disallow division operators explicitly at beginning of regular expression
    // https://eslint.org/docs/rules/no-div-regex
    'no-div-regex': OFF,

    // disallow else after a return in an if
    // https://eslint.org/docs/rules/no-else-return
    'no-else-return': [ERROR, {
      allowElseIf: false
    }],

    // disallow empty functions, except for standalone funcs/arrows
    // https://eslint.org/docs/rules/no-empty-function
    'no-empty-function': [ERROR, {
      allow: [
        // 'asyncFunctions',
        // 'asyncMethods',
        'arrowFunctions',
        // 'constructors',
        'functions',
        // 'generatorFunctions',
        // 'generatorMethods',
        // 'getters',
        'methods'
        // 'setters'
      ]
    }],

    // disallow empty destructuring patterns
    // https://eslint.org/docs/rules/no-empty-pattern
    'no-empty-pattern': ERROR,

    // disallow comparisons to null without a type-checking operator
    // If you want to enforce type-checking operations in general, use the more powerful eqeqeq instead.
    // https://eslint.org/docs/rules/no-eq-null
    'no-eq-null': OFF,

    // disallow use of eval()
    // https://eslint.org/docs/rules/no-eval
    'no-eval': ERROR,

    // disallow adding to native types
    // https://eslint.org/docs/rules/no-extend-native
    'no-extend-native': ERROR,

    // disallow unnecessary function binding
    // https://eslint.org/docs/rules/no-extra-bind
    'no-extra-bind': ERROR,

    // disallow Unnecessary Labels
    // https://eslint.org/docs/rules/no-extra-label
    'no-extra-label': ERROR,

    // disallow fallthrough of case statements
    // https://eslint.org/docs/rules/no-fallthrough
    'no-fallthrough': ERROR,

    // disallow the use of leading or trailing decimal points in numeric literals
    // https://eslint.org/docs/rules/no-floating-decimal
    'no-floating-decimal': ERROR,

    // disallow reassignments of native objects or read-only globals
    // https://eslint.org/docs/rules/no-global-assign
    'no-global-assign': [ERROR, {
      exceptions: []
    }],

    // DEPRECATED in favor of no-global-assign
    // https://eslint.org/docs/rules/no-native-reassign
    'no-native-reassign': OFF,

    // disallow implicit type conversions
    // https://eslint.org/docs/rules/no-implicit-coercion
    'no-implicit-coercion': [WARN, {
      boolean: true,
      number: true,
      string: true,
      disallowTemplateShorthand: false,
      allow: []
    }],

    // disallow var and named functions in global scope
    // https://eslint.org/docs/rules/no-implicit-globals
    'no-implicit-globals': [OFF, {
      lexicalBindings: false
    }],

    // disallow use of eval()-like methods
    // https://eslint.org/docs/rules/no-implied-eval
    'no-implied-eval': ERROR,

    // disallow this keywords outside of classes or class-like objects
    // https://eslint.org/docs/rules/no-invalid-this
    'no-invalid-this': [ERROR, {
      capIsConstructor: true
    }],

    // disallow usage of __iterator__ property
    // https://eslint.org/docs/rules/no-iterator
    'no-iterator': ERROR,

    // disallow use of labels for anything other then loops and switches
    // https://eslint.org/docs/rules/no-labels
    'no-labels': [ERROR, {
      allowLoop: false,
      allowSwitch: false
    }],

    // disallow unnecessary nested blocks
    // https://eslint.org/docs/rules/no-lone-blocks
    'no-lone-blocks': ERROR,

    // disallow creation of functions within loops
    // https://eslint.org/docs/rules/no-loop-func
    'no-loop-func': ERROR,

    // disallow magic numbers
    // https://eslint.org/docs/rules/no-magic-numbers
    'no-magic-numbers': [WARN, {
      ignore: [0, 1, 2],
      ignoreArrayIndexes: true,
      ignoreDefaultValues: true,
      enforceConst: true,
      detectObjects: false
    }],

    // disallow use of multiple spaces
    // https://eslint.org/docs/rules/no-multi-spaces
    'no-multi-spaces': [ERROR, {
      ignoreEOLComments: true,
      exceptions: {
        Property: true,
        BinaryExpression: false,
        VariableDeclarator: true,
        ImportDeclaration: true
      }
    }],

    // disallow use of multiline strings
    // https://eslint.org/docs/rules/no-multi-str
    'no-multi-str': ERROR,

    // disallow use of new operator when not part of the assignment or comparison
    // https://eslint.org/docs/rules/no-new
    'no-new': ERROR,

    // disallow use of new operator for Function object
    // https://eslint.org/docs/rules/no-new-func
    'no-new-func': ERROR,

    // disallows creating new instances of String, Number, and Boolean
    // https://eslint.org/docs/rules/no-new-wrappers
    'no-new-wrappers': ERROR,

    // disallow \8 and \9 escape sequences in string literals
    // https://eslint.org/docs/rules/no-nonoctal-decimal-escape
    'no-nonoctal-decimal-escape': ERROR,

    // disallow use of (old style) octal literals
    // https://eslint.org/docs/rules/no-nonoctal-decimal-escape
    'no-octal': ERROR,

    // disallow use of octal escape sequences in string literals, such as
    // var foo = 'Copyright \251';
    // https://eslint.org/docs/rules/no-octal-escape
    'no-octal-escape': ERROR,

    // disallow reassignment of function parameters
    // disallow parameter object manipulation except for specific exclusions
    // https://eslint.org/docs/rules/no-param-reassign
    'no-param-reassign': [ERROR, {
      props: true,
      ignorePropertyModificationsFor: [
        'acc',          // for reduce accumulators
        'accumulator',  // for reduce accumulators
        'e',            // for e.returnvalue
        'ctx',          // for Koa routing
        'context',      // for Koa routing
        'req',          // for Express requests
        'request',      // for Express requests
        'res',          // for Express responses
        'response',     // for Express responses
        'staticContext' // for ReactRouter context
      ]
    }],

    // disallow usage of __proto__ property
    // https://eslint.org/docs/rules/no-proto
    'no-proto': ERROR,

    // disallow declaring the same variable more then once
    // https://eslint.org/docs/rules/no-redeclare
    'no-redeclare': [ERROR, {
      builtinGlobals: true
    }],

    // disallow certain object properties
    // https://eslint.org/docs/rules/no-restricted-properties
    'no-restricted-properties': [ERROR, {
      object: 'arguments',
      property: 'callee',
      message: 'arguments.callee is deprecated'
    }, {
      object: 'global',
      property: 'isFinite',
      message: 'Please use Number.isFinite instead'
    }, {
      object: 'self',
      property: 'isFinite',
      message: 'Please use Number.isFinite instead'
    }, {
      object: 'window',
      property: 'isFinite',
      message: 'Please use Number.isFinite instead'
    }, {
      object: 'global',
      property: 'isNaN',
      message: 'Please use Number.isNaN instead'
    }, {
      object: 'self',
      property: 'isNaN',
      message: 'Please use Number.isNaN instead'
    }, {
      object: 'window',
      property: 'isNaN',
      message: 'Please use Number.isNaN instead'
    }, {
      property: '__defineGetter__',
      message: 'Please use Object.defineProperty instead.'
    }, {
      property: '__defineSetter__',
      message: 'Please use Object.defineProperty instead.'
    }, {
      object: 'Math',
      property: 'pow',
      message: 'Use the exponentiation operator (**) instead.'
    }],

    // disallow use of assignment in return statement
    // https://eslint.org/docs/rules/no-return-assign
    'no-return-assign': [ERROR, 'always'],

    // disallow redundant `return await`
    // https://eslint.org/docs/rules/no-return-await
    'no-return-await': ERROR,

    // disallow use of `javascript:` urls.
    // https://eslint.org/docs/rules/no-script-url
    'no-script-url': ERROR,

    // disallow self assignment
    // https://eslint.org/docs/rules/no-self-assign
    'no-self-assign': [ERROR, {
      props: true
    }],

    // disallow comparisons where both sides are exactly the same
    // https://eslint.org/docs/rules/no-self-compare
    'no-self-compare': ERROR,

    // disallow use of comma operator
    // https://eslint.org/docs/rules/no-sequences
    'no-sequences': ERROR,

    // restrict what can be thrown as an exception
    // https://eslint.org/docs/rules/no-throw-literal
    'no-throw-literal': ERROR,

    // disallow unmodified conditions of loops
    // https://eslint.org/docs/rules/no-unmodified-loop-condition
    'no-unmodified-loop-condition': ERROR,

    // disallow usage of expressions in statement position
    // https://eslint.org/docs/rules/no-unused-expressions
    'no-unused-expressions': [ERROR, {
      allowShortCircuit: true,
      allowTernary: true,
      allowTaggedTemplates: false,
      enforceForJSX: true
    }],

    // disallow unused labels
    // https://eslint.org/docs/rules/no-unused-labels
    'no-unused-labels': ERROR,

    // disallow unnecessary .call() and .apply()
    // https://eslint.org/docs/rules/no-useless-call
    'no-useless-call': ERROR,

    // disallow unnecessary catch clauses
    // https://eslint.org/docs/rules/no-useless-catch
    'no-useless-catch': ERROR,

    // disallow useless string concatenation
    // https://eslint.org/docs/rules/no-useless-concat
    'no-useless-concat': ERROR,

    // disallow unnecessary string escaping
    // https://eslint.org/docs/rules/no-useless-escape
    'no-useless-escape': ERROR,

    // disallow redundant return; keywords
    // https://eslint.org/docs/rules/no-useless-return
    'no-useless-return': ERROR,

    // disallow use of void operator
    // https://eslint.org/docs/rules/no-void
    'no-void': [ERROR, {
      allowAsStatement: false
    }],

    // disallow usage of configurable warning terms in comments: e.g. todo
    // https://eslint.org/docs/rules/no-warning-comments
    'no-warning-comments': [OFF, {
      terms: ['todo', 'fixme', 'xxx'],
      location: 'start'
    }],

    // disallow use of the with statement
    // https://eslint.org/docs/rules/no-with
    'no-with': ERROR,

    // require using Error objects as Promise rejection reasons
    // https://eslint.org/docs/rules/prefer-promise-reject-errors
    'prefer-promise-reject-errors': [ERROR, {
      allowEmptyReject: false
    }],

    // Suggest using named capture group in regular expression
    // https://eslint.org/docs/rules/prefer-named-capture-group
    'prefer-named-capture-group': WARN,

    // Prefer Object.hasOwn() over Object.prototype.hasOwnProperty.call()
    // https://eslint.org/docs/rules/prefer-object-has-own
    'prefer-object-has-own': ERROR,

    // https://eslint.org/docs/rules/prefer-regex-literals
    'prefer-regex-literals': [ERROR, {
      disallowRedundantWrapping: true
    }],

    // require use of the second argument for parseInt()
    // https://eslint.org/docs/rules/radix
    radix: [ERROR, 'always'],

    // require `await` in `async function`
    // https://eslint.org/docs/rules/require-await
    'require-await': WARN,

    // enforce the use of u flag on RegExp
    // https://eslint.org/docs/rules/require-unicode-regexp
    'require-unicode-regexp': WARN,

    // require to declare all vars on top of their containing scope
    // https://eslint.org/docs/rules/vars-on-top
    'vars-on-top': ERROR,

    // require immediate function invocation to be wrapped in parentheses
    // https://eslint.org/docs/rules/wrap-iife
    'wrap-iife': [ERROR, 'outside', {
      functionPrototypeMethods: false
    }],

    // require or disallow Yoda conditions
    // https://eslint.org/docs/rules/yoda
    yoda: [ERROR, 'never', {
      exceptRange: false
    }]
  }
};
